#!/usr/bin/env python 
from network_proxy import NetworkProxy
from file_proxy import FileProxy
from std_proxy import StdProxy
from db_proxy import DBProxy
import util

import socket
import thread
import sys
import os
import subprocess
import io
import base64

def main(pname, pargs, ports, files, dbInfo, configFile):
    print "Protecting " + pname
    protections = []
    protections.append(StdProxy)

    config = {}
    config['configFile'] = base64.b64encode(configFile, "_-")
    config['PID'] = os.getpid()
    if len(ports) > 0:
        config['PORTS'] = ports
        protections.append(NetworkProxy)
    if len(files) > 0:
        config['FILES'] = files
        protections.append(FileProxy)
    if len(dbInfo) > 0:
        config['DBHOST'] = dbInfo.split(":")[0]
        config['DBPORT'] = dbInfo.split(":")[1]
        config['DBUSER'] = dbInfo.split(":")[2]
        config['DBPASS'] = dbInfo.split(":")[3]
        protections.append(DBProxy)

    #protections = [FileProxy]
    pi = []
    spArgs = {}
    for pc in protections:
        p = pc(config)
        spArgs = dict(spArgs.items() + p.spArgs().items())
        pi.append(p)

    startedPI = []
    try:
        for p in pi:
            thread.start_new_thread(p.run, ())
            startedPI.append(p)
            
        retCode = 22
        while retCode != 0:
            retCode = subprocess.call(pargs, **spArgs)
            if retCode != 0:
                spArgs = {}
                for p in startedPI:
                    p.reportCrash()
                    spArgs = dict(spArgs.items() + p.spArgs().items())
                print "Restarting crashed application (Error Code: %d)" % (retCode)
                
    finally:
        for p in startedPI:
            p.running = False
            p.clean()

            
if __name__ == "__main__":
    if len(sys.argv) > 1:
        configFile = sys.argv[1]
        configData = file(configFile).read().split("\n")
        os.chdir(os.path.dirname(os.path.realpath(configFile)))

        program = ""
        ports = []
        files = []
        dbInfo = ""
        for l in configData:
            if l.strip().startswith("#"):
                continue
            if l.lower() == "[ports]":
                mode = 'P'
            elif l.lower() == "[files]":
                mode = 'F'
            elif l.lower() == "[database]":
                mode = 'D'
            elif l.lower() == "[program]":
                mode = 'X'
            else:
                if mode == 'X' and l != "": program += l
                l = l.strip()
                if mode == 'P' and l != "": ports.append(l)
                if mode == 'F' and l != "": files.append(l)
                if mode == 'D' and l != "": dbInfo = l
                
        pname = program.split(" ")[0]
        pargs = program.split(" ")
        print "Program Name: %s" % (pname)
        print "Args: %s" % (pargs[1:])
        print "Ports: %s" % (str(ports))
        print "Files: %s" % (str(files))
        print "Database: %s" % (dbInfo)

        main(pname, pargs, ports, files, dbInfo, configFile)
