# Initially written by Ricardo Pascal
# http://voorloopnul.com/blog/a-python-proxy-in-less-than-100-lines-of-code/
# Distributed over IDC(I Don't Care) license
# Significantly reworked by Andres Erdataen <andres@krutt.org>
# Changes for 6.858 by Tom Boning

import logging
import time
import socket
import select
import sys
import thread
import cgi
import json
import os.path

TIMELIMIT = 1.
REFRESHRATE = 1.
TCP_BACKLOG = 200
SIZELIMIT=1<<12

def pL(*args):
    return ' '.join(str(s) for s in args)

class TCPFilter:
    input_list = []
    forward_list = []
    channel = {}
    timeout = {} # client-side sockets have time limits
    bytes_remaining = {}
    # Checks whether the threading should die
    running = True
    last_request = None
    blocked_requests = {}

    def __init__(self, (host, port), (forward_host, forward_port)):
        logging.basicConfig(level=logging.DEBUG, format='%(asctime)s: %(message)s')
        logging.getLogger().setLevel(logging.INFO)
        print 'Created TCP Proxy'
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server.bind((host, port))
        self.server.listen(TCP_BACKLOG)
        self.forward_to = (forward_host, forward_port)

    def run(self):
        self.input_list.append(self.server)
        while 1:
            if not self.running:
                # Clean up?
                return
            for sock in select.select(self.input_list, [], [], REFRESHRATE)[0]:
                if sock == self.server:
                    self.on_accept()
                else:
                    data = sock.recv(1+SIZELIMIT)
                    if not data:
                        self.on_close(sock)
                    else:
                        self.on_recv(sock, data)

    def on_accept(self):
        sock, clientaddr = self.server.accept()
        logging.debug(pL(clientaddr, "has connected"))
        try:
            forward = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            forward.connect(self.forward_to)
        except Exception, e:
            logging.warn("The filtered service seems to be down; dropping client")
            logging.warn(e)
            #sock.close()
            return
        self.input_list.append(sock)
        self.input_list.append(forward)
        self.forward_list.append(forward)
        self.channel[sock] = forward
        self.channel[forward] = sock
        self.timeout[sock] = time.time() + TIMELIMIT
        self.bytes_remaining[sock] = SIZELIMIT

    def on_close(self, sock):
        logging.debug(pL(sock.getpeername(), "has disconnected"))
        self.channel[sock].close()
        sock.close()
        self.input_list.remove(self.channel[sock])
        if self.channel[sock] in self.forward_list:
            self.forward_list.remove(self.channel[sock])
        self.input_list.remove(sock)
        if sock in self.timeout: del self.timeout[sock]
        if sock in self.bytes_remaining: del self.bytes_remaining[sock]
        del self.channel[self.channel[sock]]
        del self.channel[sock]

    def on_recv(self, sock, data):
        details = ['from', sock.getpeername(), 'to', self.channel[sock].getpeername(),repr(data), '-', len(data), 'bytes of', self.bytes_remaining.get(sock, 1e333), 'remaining']
        #print 'RECEIVED REQUEST:'
        #print 'sock in forward_list:', sock in self.forward_list
        if (sock in self.timeout and self.check_data(sock,data)):
            logging.debug(pL(*(details + ['(rejected)'])))
            self.on_close(sock)
            return
        else:
            logging.debug(pL(*details))
        if sock in self.bytes_remaining: 
            self.bytes_remaining[sock] -= len(data)
            if sock in self.forward_list: # From Server
                self.channel[sock].send(data)
            else:
                self.last_request = data
                self.channel[sock].send(self.filter_data(sock, data))
        #self.channel[sock].send(data)

    def check_data(self, sock, data):
        if data in self.blocked_requests:
            if self.blocked_requests[data] > 2:
                return True
        return False
        # Could also be something like the following, depending on the app.
        # return any([
        #     len(data) > self.bytes_remaining.get(sock, 1e333),
        #     'AAAAA' in data.upper(),
        #     'SELECT' in data.upper(),
        #     'DROP' in data.upper()
        #     ])
    def filter_data(self, sock, data):
        # Could strip out damaging inputs, such as XSS or SQL injection attempts.
        return data

    def block_last_request(self):
        if self.last_request:
            #print 'BLOCKING REQUST:%s' % self.last_request
            if self.last_request in self.blocked_requests:
                self.blocked_requests[self.last_request] += 1
            else:
                self.blocked_requests[self.last_request] = 1

    def load_blocked_requests(self,file_name):
        if not os.path.isfile(file_name):
            return
        try:
            with open(file_name, 'r') as outfile:
                loaded = json.load(outfile)
                self.blocked_requests = loaded
        except Exception, e:
            raise e

    def dump_blocked_requests(self,file_name):
        with open(file_name, 'w') as outfile:
            json.dump(self.blocked_requests, outfile)


