import socket
import subprocess
import sys
import os

FNULL = open(os.devnull, 'w')

#Find a random port that hasn't been bound.
def randomPort():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    addr, port = s.getsockname()
    s.close()
    return port
    
#Generate a random port and then create an iptables rule for it.
def forward(src):
    port = randomPort()
    forwardPort(src, port)
    return port

#Add an iptables rule.
def forwardPort(src, dst):
    subprocess.call(["iptables", "-t", "nat", "-A", "PREROUTING", "-p", "tcp", "--dport", str(src), "-j", "REDIRECT", "--to-port", str(dst)])
  
#Remove the iptables rule.
def destroyForward(src, dst):
    subprocess.call(["iptables", "-t", "nat", "-D", "PREROUTING", "-p", "tcp", "--dport", str(src), "-j", "REDIRECT", "--to-port", str(dst)])

#Watch the file using the linux audit system.
def watch(f):
    tag = f + "-watch"
    subprocess.call(["auditctl", "-w", f, "-k", tag])

#Stop watching the file with tag.
def unwatch(f):
    tag = f + "-watch"
    subprocess.call(["auditctl", "-W", f, "-k", tag])

#Returns all the audit logs for tag.
def getWatch(f):
    tag = f + "-watch"
    data = subprocess.Popen(["ausearch", "-k", tag, "-ts", "recent"], stdin=sys.stderr, stdout=subprocess.PIPE, stderr=FNULL).communicate()[0]
    return data

#Returns the last audit log for tag.
def getLastWatch(tag):
    data = getWatch(tag).strip().split("\n")[::-1]
    lastLog = []
    for l in data:
        lastLog.append(l)
        if l.startswith("time->"):
            return lastLog[::-1]
